package org.example;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

import static org.example.Modes.*;


public class Main extends JFrame {
    private ArrayList<Primitive> primitives = new ArrayList<>();
    private Modes mode = null;

    private Primitive currentPrimitive = null;

    private JLabel convexityLabel;


    private int alpha;

    private Polygon polygon;

    public Main() {
        setTitle("App");
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(800, 600);
        buildPanel();
        setVisible(true);
        convexityLabel = new JLabel("");
        add(convexityLabel, BorderLayout.SOUTH);
    }


    public void buildPanel() {
        JPanel buttonPanel = new JPanel();
        addMainButtons(buttonPanel);
        JPopupMenu menu = new JPopupMenu();
        addMoveActionsButtons(menu);
        addMouseListener(menu);
    }


    public void addMouseListener(JPopupMenu menu) {
        addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                int x = e.getX();
                int y = e.getY();
                if (mode == isRotatingAroundPoint) {
                   rotatePolygon(x, y);
                }
                else if (mode == isAddingPolygon) {
                    if(polygon == null || polygon.filled) {
                       initPolygon(x, y);
                    }else {
                        addEdgeToPolygon(x, y, menu);
                    }
                } else if (mode == isAddingEdge) {
                    if (currentPrimitive == null || currentPrimitive.filled) {
                        initEdge(x,y);
                    } else {
                       paintEdge(x, y);
                    }
                } else if (mode == isSelectingPoint) {
                    Primitive point = new Primitive(x, y);
                    primitives.add(point);
                    repaint();
                }else if(mode == isInPrimitive) {
                    if (isPointInsidePolygon(new Point(x, y), polygon)) {
                        JOptionPane.showMessageDialog(null, "Точка лежит внутри многоугольника.");
                    } else {
                        JOptionPane.showMessageDialog(null, "Точка не лежит внутри многоугольника.");
                    }
                }
                else if (mode == isPointOnEdge) {
                    PointPosition position = classifyPointPosition(new Point(x, y), currentPrimitive);
                    if (position == PointPosition.ON_EDGE) {
                        JOptionPane.showMessageDialog(null, "Точка лежит на ребре.");
                    } else if (position == PointPosition.RIGHT_OF_EDGE) {
                        JOptionPane.showMessageDialog(null, "Точка справа от ребра.");
                    } else {
                        JOptionPane.showMessageDialog(null, "Точка слева от ребра.");
                    }
                }
            }
        });
    }


    public void rotatePolygon(int x, int y) {
        Primitive point = new Primitive(x, y);
        point.setColor(Color.RED);
        primitives.add(point);
        repaint();
        polygon.rotate(x, y, alpha);
        repaint();
    }

    public void paintEdge(int x, int y) {
        if(Math.abs(currentPrimitive.points.get(0).y - y) <= 15){
            y = currentPrimitive.points.get(0).y;
        }
        currentPrimitive.points.add(new Point(x, y));

        currentPrimitive.calculateCenter();
        primitives.add(new Primitive(currentPrimitive.center));
        currentPrimitive.filled = true;
        repaint();
    }

    public void initEdge(int x, int y) {
        currentPrimitive = new Edge(x, y);
        primitives.add(currentPrimitive);
        repaint();
    }

    public void paintIntersect(Point p) {
        currentPrimitive = new Primitive((int) p.getX(), (int) p.getY());
        primitives.add(currentPrimitive);
        repaint();

    }

    public void initPolygon(int x, int y) {
        polygon = new Polygon(x, y);
        currentPrimitive = polygon;
        primitives.add(polygon);
        repaint();
    }

    public void addEdgeToPolygon(int x, int y, JPopupMenu menu) {
        if (polygon.points.size() > 1) {
            Point firstPoint = new Point(x, y);
            Point lastPoint = polygon.points.get(0);
            if (Math.abs(firstPoint.x - lastPoint.x) <= 10 && Math.abs(firstPoint.y - lastPoint.y) <= 10) {
                polygon.filled = true;
                polygon.calculateCenter();

                if (isConvexPolygon(polygon)) {
                    convexityLabel.setText("Многоугольник выпуклый.");
                } else {
                    convexityLabel.setText("Многоугольник невыпуклый.");
                }

                repaint();
                menu.show(Main.this, x, y);
            } else {
                Point point = new Point(x, y);
                polygon.addPoint(point);
                repaint();
            }
        } else {
            Point point = new Point(x, y);
            polygon.addPoint(point);
            repaint();
        }
    }

    public void addMoveActionsButtons(JPopupMenu menu) {
        JMenuBar menuBar = new JMenuBar();
        setJMenuBar(menuBar);

        JMenu actionMenu = new JMenu("Действия");
        JMenu actionMenu2 = new JMenu("Проверки");
        menuBar.add(actionMenu2);
        menuBar.add(actionMenu);

        JMenuItem isInPrimitiveItem = new JMenuItem("Лежит ли точка в многоугольнике");
        JMenuItem pointOnEdge = new JMenuItem("Точка относительно ребра");

        JMenuItem moveMenuItem = new JMenuItem("Переместить");
        JMenuItem rotateAroundPointMenuItem = new JMenuItem("Повернуть вокруг точки");
        JMenuItem rotateAroundCenterMenuItem = new JMenuItem("Повернуть вокруг центра");
        JMenuItem scaleCenterMenuItem = new JMenuItem("Масштабировать примитив");
        JMenuItem intersectionMenuItem = new JMenuItem("Найти пересечение ребер");


        pointOnEdge.addActionListener(e -> {
            if (currentPrimitive instanceof Edge) {
                mode = isPointOnEdge;
            }
        });

        isInPrimitiveItem.addActionListener(e -> {
            if (currentPrimitive instanceof Polygon) {
                mode = isInPrimitive;
            }
        });

        moveMenuItem.addActionListener(e -> {
            String newXStr = JOptionPane.showInputDialog("Введите сдвиг по X:");
            String newYStr = JOptionPane.showInputDialog("Введите сдвиг по Y:");

            try {
                int newX = Integer.parseInt(newXStr);
                int newY = Integer.parseInt(newYStr);
                currentPrimitive.translate(newX, newY);
                repaint();
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(null, "Неверный формат ввода.");
            }
        });

        rotateAroundPointMenuItem.addActionListener(e -> {
            mode = isRotatingAroundPoint;
            String alphaStr = JOptionPane.showInputDialog("Введите угол:");
            try {
                alpha = Integer.parseInt(alphaStr);
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(null, "Неверный формат ввода.");
            }
        });

        scaleCenterMenuItem.addActionListener(e -> {
            mode = isScaling;
            String koeffXStr = JOptionPane.showInputDialog("Введите коэффициент по X:");
            String koeffYStr = JOptionPane.showInputDialog("Введите коэффициент по Y:");
            try {
                double koeffX = Double.parseDouble(koeffXStr);
                double koeffY = Double.parseDouble(koeffYStr);
                currentPrimitive.scale(koeffX, koeffY);
                repaint();
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(null, "Неверный формат ввода.");
            }
        });

        rotateAroundCenterMenuItem.addActionListener(e -> {
            mode = isRotatingAroundCenter;
            String alphaStr = JOptionPane.showInputDialog("Введите угол:");
            try {
                int alpha = Integer.parseInt(alphaStr);
                currentPrimitive.rotateAroundCenter(alpha);
                repaint();
            } catch (NumberFormatException ex) {
                JOptionPane.showMessageDialog(null, "Неверный формат ввода.");
            }
        });

        intersectionMenuItem.addActionListener(e -> {
            try {
                Edge edge1 = (Edge) primitives.get(primitives.size() - 2);
                Edge edge2 = (Edge) primitives.get(primitives.size() - 4);
                Point interP = edge2.findIntersecting(edge1);
                if (interP != null)
                    paintIntersect(interP);
                repaint();
            } catch (Error ex) {
                JOptionPane.showMessageDialog(null, ex.getMessage());
            }
        });

        actionMenu.add(moveMenuItem);
        actionMenu.add(rotateAroundPointMenuItem);
        actionMenu.add(rotateAroundCenterMenuItem);
        actionMenu.add(scaleCenterMenuItem);
        actionMenu2.add(isInPrimitiveItem);
        actionMenu2.add(pointOnEdge);
        actionMenu.add(intersectionMenuItem);
    }

    public void addMainButtons(JPanel buttonPanel) {
        JButton pointButton = new JButton("Point");
        JButton edgeButton = new JButton("Edge");
        JButton polygonButton = new JButton("Polygon");
        JButton clearButton = new JButton("Clear");

        pointButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int x = 300;
                int y = 300;
                Primitive point = new Primitive(x, y);
                primitives.add(point);
                repaint();
                mode = isSelectingPoint;
            }
        });

        edgeButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mode = isAddingEdge;

            }
        });

        polygonButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                mode = isAddingPolygon;
            }
        });
        clearButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                primitives.clear();
                mode = null;
                currentPrimitive = null;
                polygon = null;
                convexityLabel.setText("");
                repaint();
            }
        });
        buttonPanel.add(pointButton);
        buttonPanel.add(edgeButton);
        buttonPanel.add(polygonButton);
        buttonPanel.add(clearButton);

        add(buttonPanel, BorderLayout.NORTH);
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        for (int i = 0; i < primitives.size(); i++) {
            primitives.get(i).draw(g);
        }
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(Main::new);
    }


    private boolean isConvexPolygon(Polygon polygon) {
        int n = polygon.points.size();
        if (n < 3) {
            return false; // Многоугольник с меньше чем 3 вершинами не считается выпуклым
        }

        boolean sign = false;
        for (int i = 0; i < n; i++) {
            int dx1 = polygon.points.get((i + 2) % n).x - polygon.points.get((i + 1) % n).x;
            int dy1 = polygon.points.get((i + 2) % n).y - polygon.points.get((i + 1) % n).y;
            int dx2 = polygon.points.get(i).x - polygon.points.get((i + 1) % n).x;
            int dy2 = polygon.points.get(i).y - polygon.points.get((i + 1) % n).y;

            int crossProduct = dx1 * dy2 - dx2 * dy1;

            if (i == 0) {
                sign = crossProduct > 0;
            } else {
                if (crossProduct > 0 != sign) {
                    return false;
                }
            }
        }

        return true;
    }

    private boolean isPointInsidePolygon(Point point, Polygon polygon) {
        int n = polygon.points.size();
        if (n < 3) {
            return false;
        }

        int minX = Integer.MAX_VALUE;
        int maxX = Integer.MIN_VALUE;
        int minY = Integer.MAX_VALUE;
        int maxY = Integer.MIN_VALUE;

        for (Point vertex : polygon.points) {
            if (vertex.x < minX) {
                minX = vertex.x;
            }
            if (vertex.x > maxX) {
                maxX = vertex.x;
            }
            if (vertex.y < minY) {
                minY = vertex.y;
            }
            if (vertex.y > maxY) {
                maxY = vertex.y;
            }
        }

        if (point.x < minX || point.x > maxX || point.y < minY || point.y > maxY) {
            return false;
        }

        boolean isInside = false;
        int j = n - 1;
        for (int i = 0; i < n; i++) {
            boolean isVertexAbove = polygon.points.get(i).y > point.y;
            boolean isPrevVertexAbove = polygon.points.get(j).y > point.y;

            boolean isIntersecting = point.x < (polygon.points.get(j).x - polygon.points.get(i).x) * (point.y - polygon.points.get(i).y) /
                    (polygon.points.get(j).y - polygon.points.get(i).y) + polygon.points.get(i).x;

            if ((isVertexAbove != isPrevVertexAbove) && isIntersecting) {
                isInside = !isInside;
            }

            j = i;
        }

        return isInside;
    }

    public enum PointPosition {
        ON_EDGE, RIGHT_OF_EDGE, LEFT_OF_EDGE
    }

    public PointPosition classifyPointPosition(Point point, Primitive edge) {
        int x1 = edge.points.get(0).x;
        int y1 = edge.points.get(0).y;
        int x2 = edge.points.get(1).x;
        int y2 = edge.points.get(1).y;

        int crossProduct = (x2 - x1) * (point.y - y1) - (y2 - y1) * (point.x - x1);

        if (crossProduct == 0) {
            return PointPosition.ON_EDGE;
        } else if (crossProduct > 0) {
            return PointPosition.RIGHT_OF_EDGE;
        } else {
            return PointPosition.LEFT_OF_EDGE;
        }
    }
}
