package org.example;

import java.awt.*;
import java.util.ArrayList;

class Primitive {
    boolean filled = false;
    public ArrayList<Point> points;
    public Point center;
    public Color color = Color.BLACK;

    public Primitive(int x, int y) {
        points = new  ArrayList<Point>();
        points.add(new Point(x,y));
    }

    public void setColor(Color color){
        this.color = color;
    }
    public Primitive(Point point) {
        points = new  ArrayList<Point>();
        points.add(point);
    }

    public void translate(int dx, int dy) {
        double[][] translationMatrix = {
                {1, 0, dx},
                {0, 1, dy},
                {0, 0, 1}
        };

        for (Point point1 : points) {
            transformPoint(translationMatrix, point1);
        }
        transformPoint(translationMatrix, center);
    }

    public void rotate(int x, int y, int alpha) {
        double cosA = Math.cos(Math.toRadians(alpha));
        double sinA = Math.sin(Math.toRadians(alpha));
        double[][] rotationMatrix = {
                {cosA, sinA, 0},
                {-sinA, cosA, 0},
                {0, 0, 1}
        };
        translate(-x, -y);
        for (Point point : points) {
            transformPoint(rotationMatrix, point);
        }
        translate(x, y);
        calculateCenter();
    }


    public void scale(double koeffX, double koeffY) {
        double[][] rotationMatrix = {
                {(double) 1 /koeffX, 0, 0},
                {0, (double) 1 /koeffY, 0},
                {0, 0, 1}
        };
        if(this.center == null){
            calculateCenter();
        }
        int x = center.x;
        int y = center.y;
        translate(-center.x, -center.y);
        for (Point point : points) {
            transformPoint(rotationMatrix, point);
        }
        translate(x, y);
        calculateCenter();
    }


    public void rotateAroundCenter(int alpha) {
        calculateCenter();
        rotate(center.x, center.y, alpha);
    }


    public void calculateCenter() {
        ArrayList<Point> points = this.points;
        int totalX = 0;
        int totalY = 0;
        for (Point point : points) {
            totalX += point.x;
            totalY += point.y;
        }
        int centerX = totalX / points.size();
        int centerY = totalY / points.size();

        this.center = new Point(centerX, centerY);
    }

    public void transformPoint(double[][] translationMatrix, Point point){
        if(point == null){
            return;
        }
        double x = point.x;
        double y = point.y;
        double newX = translationMatrix[0][0] * x + translationMatrix[0][1] * y + translationMatrix[0][2];
        double newY = translationMatrix[1][0] * x + translationMatrix[1][1] * y + translationMatrix[1][2];
        point.x = (int) newX;
        point.y = (int) newY;
    }

    public void draw(Graphics g) {
        g.setColor(color);
        int radius = 5 / 2;
        g.fillOval(points.get(0).x - radius, points.get(0).y - radius, 5, 5);
    }

}


class Edge extends Primitive {
    public Edge(int x1, int y1, int x2, int y2) {
        super(x1, y1);
        points.add(new Point(x2,y2));
    }

    public Edge(int x1, int y1) {
        super(x1, y1);
    }

    public Point getLastPoint(){
        return this.points.get(points.size()-1);
    }


    public Point findIntersecting(Edge edge) {
        Point res;
        double x1 = this.points.get(0).getX();
        double y1 = this.points.get(0).getY();
        double x2 = this.points.get(1).getX();
        double y2 = this.points.get(1).getY();
        double x3 = edge.points.get(0).getX();
        double y3 = edge.points.get(0).getY();
        double x4 = edge.points.get(1).getX();
        double y4 = edge.points.get(1).getY();

        double denominator = ((x1 - x2) * (y3 - y4)) - ((y1 - y2) * (x3 - x4));

        if (denominator == 0) {
            res = null;
        }

        double x = (((x1 * y2) - (y1 * x2)) * (x3 - x4) - (x1 - x2) * ((x3 * y4) - (y3 * x4))) / denominator;
        double y = (((x1 * y2) - (y1 * x2)) * (y3 - y4) - (y1 - y2) * ((x3 * y4) - (y3 * x4))) / denominator;
        //if (((x-x1)*(y2-y1)-(y-y1)*(x2-x1) !=0) || !(((x1>x) &&(x2<x)) ||((x1<x) &&(x2>x)) ))
        if (x>Math.max(x1,x2)||x>Math.max(x3,x4)||x<Math.min(x3,x4)||x<Math.min(x1,x2)
            ||y>Math.max(y1,y2)||y>Math.max(y3,y4)||y<Math.min(y3,y4)||y<Math.min(y1,y2))
            res=null;
        else
            res = new Point((int) x, (int) y);
        return res;
    }

    @Override
    public void draw(Graphics g) {
        g.setColor(Color.BLACK);
        int radius = 5 / 2;
        Point point1 = points.get(0);
        g.fillOval(point1.x-radius, point1.y -radius , 5, 5);
        if(points.size() > 1) {
            Point point2 = points.get(1);
            g.drawLine(point1.x, point1.y, point2.x, point2.y);
            g.fillOval(point2.x-radius, point2.y -radius , 5, 5);
        }
    }
}

class Polygon extends Primitive{

    public Polygon(int x, int y) {
        super(x, y);
    }

    public void addPoint(int x, int y){
        points.add(new Point(x,y));
    }

    public void addPoint(Point point){
        points.add(point);
    }


    @Override
    public void draw(Graphics g) {
        g.setColor(Color.BLACK);
        int radius = 5 / 2;
        Point point1 = points.get(0);
        Point point2 = null;
        g.fillOval(point1.x-radius, point1.y -radius , 5, 5);
        for(int i = 1; i <= points.size()-1; i++){
            point1 = points.get(i);
            point2 = points.get(i-1);
            g.fillOval(point1.x-radius, point1.y -radius , 5, 5);
            g.drawLine(point1.x, point1.y, point2.x, point2.y);
        }
        if(filled) {
            point2 = points.get(0);
            g.drawLine(point1.x, point1.y, point2.x, point2.y);
            g.fillOval(center.x-radius, center.y -radius , 5, 5);
        }



    }

}
